"use strict";
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var _errors_1 = require("./_errors");
var _results_1 = require("./_results");
var stream_1 = require("stream");
var TestOutputStream = (function (_super) {
    __extends(TestOutputStream, _super);
    function TestOutputStream() {
        _super.apply(this, arguments);
    }
    TestOutputStream.prototype._read = function () { };
    TestOutputStream.prototype.end = function () {
        this.push(null);
    };
    TestOutputStream.prototype._writeOut = function (message) {
        this.push(message);
    };
    TestOutputStream.prototype.emitVersion = function () {
        this._writeOut("TAP version 13\n");
    };
    TestOutputStream.prototype.emitPlan = function (testCount) {
        this._writeOut("1.." + testCount + "\n");
    };
    TestOutputStream.prototype.emitFixture = function (fixture) {
        this._writeOut("# FIXTURE " + fixture.description + "\n");
    };
    TestOutputStream.prototype.emitResult = function (testId, result) {
        var outcome = result.outcome;
        var test = result.test;
        var testCaseArguments = result.arguments;
        if (outcome === _results_1.TestOutcome.Pass) {
            this._emitPass(testId, test, testCaseArguments);
        }
        else if (outcome === _results_1.TestOutcome.Fail || outcome === _results_1.TestOutcome.Error) {
            var error = result.error;
            this._emitFail(testId, test, testCaseArguments, error);
        }
        else if (outcome === _results_1.TestOutcome.Skip) {
            this._emitSkip(testId, test, testCaseArguments);
        }
        else {
            throw new TypeError("Invalid test outcome: " + outcome);
        }
    };
    TestOutputStream.prototype._emitPass = function (testId, test, testCaseArguments) {
        var description = this._getTestDescription(test, testCaseArguments);
        this._writeOut("ok " + testId + " " + description + "\n");
    };
    TestOutputStream.prototype._emitSkip = function (testId, test, testCaseArguments) {
        var description = this._getTestDescription(test, testCaseArguments);
        // we only want to use the reason if it's not undefined
        var reasonString = "";
        if (test.ignoreReason !== undefined) {
            reasonString = " " + test.ignoreReason;
        }
        this._writeOut("ok " + testId + " " + description + " # skip" + reasonString + "\n");
    };
    TestOutputStream.prototype._emitFail = function (testId, test, testCaseArguments, error) {
        var description = this._getTestDescription(test, testCaseArguments);
        this._writeOut("not ok " + testId + " " + description + "\n");
        if (error instanceof _errors_1.MatchError === false) {
            error = new _errors_1.MatchError("the test threw an error", "the test to run", "Test threw " + error.constructor.name + " with message \"" + error.message + "\"");
        }
        this._writeOut(this._getErrorYaml(error));
    };
    TestOutputStream.prototype._getTestDescription = function (test, testCaseArguments) {
        var _this = this;
        var testDescription = test.description;
        if (testCaseArguments !== undefined && testCaseArguments.length > 0) {
            testDescription += " [ " + testCaseArguments.map(function (argument) { return _this._getArgumentDescription(argument); }).join(", ") + " ]";
        }
        return testDescription;
    };
    TestOutputStream.prototype._getArgumentDescription = function (argument) {
        var jsonArgument = JSON.stringify(argument);
        // if the argument can be expresed as JSON return that
        if (jsonArgument) {
            return JSON.stringify(argument);
        }
        else if (argument && argument.name) {
            return argument.name;
        }
        else if (argument instanceof Function) {
            return "anonymous function";
        }
        // otherwise must be undefined
        return "undefined";
    };
    TestOutputStream.prototype._getErrorYaml = function (error) {
        return " ---\n" +
            "   message: \"" + error.message.replace(/\\/g, "\\\\").replace(/"/g, "\\\"") + "\"\n" +
            "   severity: fail\n" +
            "   data:\n" +
            "     got: " + JSON.stringify(error.actualValue) + "\n" +
            "     expect: " + JSON.stringify(error.expectedValue) + "\n ...\n";
    };
    return TestOutputStream;
}(stream_1.Readable));
exports.TestOutputStream = TestOutputStream;
//# sourceMappingURL=test-output-stream.js.map